@@ -194,53 +194,87 @@ static void powered_changed(struct connman_device *device)
 					DBUS_TYPE_BOOLEAN, &device->powered);
 }
 
-static int set_powered(struct connman_device *device, connman_bool_t powered)
+static void blocked_changed(struct connman_device *device)
+{
+	connman_dbus_property_changed_basic(device->element.path,
+				CONNMAN_DEVICE_INTERFACE, "Blocked",
+					DBUS_TYPE_BOOLEAN, &device->blocked);
+}
+
+int __connman_device_enable(struct connman_device *device)
 {
-	struct connman_device_driver *driver = device->driver;
 	int err;
 
-	DBG("device %p powered %d", device, powered);
+	DBG("device %p", device);
 
-	if (device->powered_pending == powered)
+	if (!device->driver || !device->driver->enable)
+		return -EOPNOTSUPP;
+
+	if (device->powered_pending == TRUE)
 		return -EALREADY;
 
-	if (!driver)
-		return -EINVAL;
+	if (device->blocked == TRUE)
+		return -ENOLINK;
 
-	if (powered == TRUE) {
-		if (driver->enable) {
-			device->powered_pending = powered;
-
-			err = driver->enable(device);
-			if (err == 0)
-				__connman_technology_enable_device(device);
-		} else
-			err = -EINVAL;
-	} else {
-		device->powered_pending = powered;
-
-		device->reconnect = FALSE;
-
-		clear_scan_trigger(device);
-
-		g_hash_table_remove_all(device->networks);
-
-		if (driver->disable) {
-			err = driver->disable(device);
-			if (err == 0)
-				__connman_technology_disable_device(device);
-		} else
-			err = -EINVAL;
+	err = device->driver->enable(device);
+	if (err < 0) {
+		if (err == -EINPROGRESS)
+			device->powered_pending = TRUE;
+		return err;
 	}
 
-	if (err == 0) {
-		device->powered = powered;
+	device->powered_pending = TRUE;
+	device->powered = TRUE;
 
-		if (device->registered == TRUE)
-			powered_changed(device);
+	__connman_technology_enable_device(device);
+
+	return 0;
+}
+
+int __connman_device_disable(struct connman_device *device)
+{
+	int err;
+
+	DBG("device %p", device);
+
+	if (!device->driver || !device->driver->disable)
+		return -EOPNOTSUPP;
+
+	if (device->powered == FALSE)
+		return -ENOLINK;
+
+	if (device->powered_pending == FALSE)
+		return -EALREADY;
+
+	device->reconnect = FALSE;
+
+	clear_scan_trigger(device);
+
+	g_hash_table_remove_all(device->networks);
+
+	err = device->driver->disable(device);
+	if (err < 0) {
+		if (err == -EINPROGRESS)
+			device->powered_pending = FALSE;
+		return err;
 	}
 
-	return err;
+	device->powered_pending = FALSE;
+	device->powered = FALSE;
+
+	__connman_technology_disable_device(device);
+
+	return 0;
+}
+
+static int set_powered(struct connman_device *device, connman_bool_t powered)
+{
+	DBG("device %p powered %d", device, powered);
+
+	if (powered == TRUE)
+		return __connman_device_enable(device);
+	else
+		return __connman_device_disable(device);
 }
 
 void __connman_device_list(DBusMessageIter *iter, void *user_data)
@@ -306,6 +340,9 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	connman_dbus_dict_append_basic(&dict, "Powered",
 					DBUS_TYPE_BOOLEAN, &device->powered);
 
+	connman_dbus_dict_append_basic(&dict, "Blocked",
+					DBUS_TYPE_BOOLEAN, &device->blocked);
+
 	if (device->driver && device->driver->scan)
 		connman_dbus_dict_append_basic(&dict, "Scanning",
 					DBUS_TYPE_BOOLEAN, &device->scanning);
@@ -962,8 +999,10 @@ int connman_device_set_powered(struct connman_device *device,
 		device->pending = NULL;
 	}
 
-	if (device->powered == powered)
+	if (device->powered == powered) {
+		device->powered_pending = powered;
 		return -EALREADY;
+	}
 
 	if (powered == TRUE)
 		__connman_device_enable(device);
@@ -978,6 +1017,11 @@ int connman_device_set_powered(struct connman_device *device,
 	else
 		__connman_technology_disable_device(device);
 
+	if (device->offlinemode == TRUE && powered == TRUE) {
+		powered_changed(device);
+		return connman_device_set_powered(device, FALSE);
+	}
+
 	if (device->registered == FALSE)
 		return 0;
 
@@ -1003,6 +1047,8 @@ int __connman_device_set_blocked(struct connman_device *device,
 
 	device->blocked = blocked;
 
+	blocked_changed(device);
+
 	if (device->offlinemode == TRUE)
 		return 0;
 
@@ -1016,6 +1062,11 @@ int __connman_device_set_blocked(struct connman_device *device,
 	return set_powered(device, powered);
 }
 
+connman_bool_t __connman_device_get_blocked(struct connman_device *device)
+{
+	return device->blocked;
+}
+
 int __connman_device_scan(struct connman_device *device)
 {
 	if (!device->driver || !device->driver->scan)
@@ -1029,74 +1080,24 @@ int __connman_device_scan(struct connman_device *device)
 	return device->driver->scan(device);
 }
 
-int __connman_device_enable(struct connman_device *device)
+int __connman_device_enable_persistent(struct connman_device *device)
 {
 	int err;
 
 	DBG("device %p", device);
 
-	if (!device->driver || !device->driver->enable)
-		return -EOPNOTSUPP;
-
-	if (device->powered_pending == TRUE)
-		return -EALREADY;
-
-	device->powered_pending = TRUE;
-
-	err = device->driver->enable(device);
-	if (err < 0)
-		return err;
-
-	device->powered = TRUE;
-
-	__connman_technology_enable_device(device);
-
-	return 0;
-}
-
-int __connman_device_enable_persistent(struct connman_device *device)
-{
-	DBG("device %p", device);
-
 	device->powered_persistent = TRUE;
 
 	__connman_storage_save_device(device);
 
-	return __connman_device_enable(device);
-}
+	err = __connman_device_enable(device);
+	if (err == 0 || err == -EINPROGRESS) {
+		device->offlinemode = FALSE;
+		if (__connman_profile_get_offlinemode() == TRUE)
+			__connman_profile_set_offlinemode(FALSE, FALSE);
+	}
 
-int __connman_device_disable(struct connman_device *device)
-{
-	int err;
-
-	DBG("device %p", device);
-
-	if (!device->driver || !device->driver->disable)
-		return -EOPNOTSUPP;
-
-	if (device->powered == FALSE)
-		return -ENOLINK;
-
-	if (device->powered_pending == FALSE)
-		return -EALREADY;
-
-	device->powered_pending = FALSE;
-
-	device->reconnect = FALSE;
-
-	clear_scan_trigger(device);
-
-	g_hash_table_remove_all(device->networks);
-
-	err = device->driver->disable(device);
-	if (err < 0)
-		return err;
-
-	device->powered = FALSE;
-
-	__connman_technology_disable_device(device);
-
-	return 0;
+	return err;
 }
 
 int __connman_device_disable_persistent(struct connman_device *device)
